#include "image_ppm.hpp"

#include "color.hpp"
#include <fstream>

#include <cstdlib>

image load_image(const std::string& filename)
{
    std::ifstream ifs(filename.c_str());

    if(ifs.good()==false)
    {
        std::cerr<<"Cannot open "<<filename<<std::endl;
        exit(1);
    }

    std::string buffer;
    std::getline(ifs,buffer);

    if(buffer!="P3")
    {
        std::cerr<<"File is not P3 type"<<std::endl;
        exit(1);
    }


    //ligne de commentaire
    std::getline(ifs,buffer);
    if(buffer.size()<1 || buffer[0]!='#')
    {
        std::cerr<<"Absence de commentaires"<<std::endl;
        exit(1);
    }

    //taille de l'image
    int Nx=-1,Ny=-1;
    ifs>>Nx;
    ifs>>Ny;

    if(Nx==-1 || Ny==-1)
    {
        std::cerr<<"Taille non lue"<<std::endl;
        exit(1);
    }

    if(Nx<1 || Nx>3000 || Ny<1 || Ny>3000)
    {
        std::cerr<<"Dimensions image aberrantes"<<std::endl;
        exit(1);
    }

    //lecture intensite max
    int max=-1;
    ifs>>max;
    if(max!=255)
    {
        std::cerr<<"max="<<max<<" != 255"<<std::endl;
        exit(1);
    }


    //allocation image
    image im;
    im.Nx=Nx;
    im.Ny=Ny;
    im.data.resize(Nx*Ny);

    //lecture des donnees
    for(int ky=0;ky<Ny;++ky)
    {
        for(int kx=0;kx<Nx;++kx)
        {
            if(ifs.good()!=true)
            {
                std::cerr<<"Fin de fichier premature"<<std::endl;
                exit(1);
            }

            //lecture valeur
            color c;
            ifs>>c.r;
            ifs>>c.g;
            ifs>>c.b;

            //verification valeur
            if(is_valid(c)!=true)
            {
                std::cerr<<"Valeur "<<c.r<<","<<c.g<<","<<c.b<<" incorrecte"<<std::endl;
                exit(1);
            }

            //affectation dans le tableau de couleur
            im(kx,ky)=c;
        }
    }


    ifs.close();

    return im;
}

void save_image(const std::string& filename,const image& im)
{
    std::ofstream ofs(filename.c_str());
    if(ofs.good()==false)
    {
        std::cerr<<"Cannot open file "<<filename<<std::endl;
        exit(1);
    }

    ofs<<"P3"<<std::endl;
    ofs<<"#Commentaire"<<std::endl;
    ofs<<im.Nx<<" "<<im.Ny<<std::endl;
    ofs<<255<<std::endl;


    for(int ky=0;ky<im.Ny;++ky)
        for(int kx=0;kx<im.Nx;++kx)
            ofs<<im(kx,ky)<<std::endl;


    ofs.close();
}
