
#include "render_area.hpp"

#include <GL/glu.h>
#include <iostream>

#include <QPainter>
#include <QMouseEvent>


render_area::render_area(QWidget *parent)
    :QWidget(parent)
{
    //fond blanc
    setBackgroundRole(QPalette::Base);
    setAutoFillBackground(true);

    //initialisation attributs d'affichage et de mode
    is_drawing_mode=true;
    is_click_mode=false;
    is_draw_inertia=true;
    is_draw_tangent=true;

}

render_area::~render_area()
{}



void render_area::paintEvent(QPaintEvent*)
{
    //la classe qui permet de "dessiner"
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing, true);

    //le pinceau de dessin
    QPen pen;


    //affichage des points sources
    pen.setWidth(2.0);
    painter.setPen(Qt::black);
    painter.setPen(pen);
    data.draw_src_vertex(painter);


    //affiche les axes d'inerties
    pen.setWidth(2.0);
    pen.setColor(Qt:: blue);
    painter.setPen(pen);
    if(is_draw_inertia)
    {
        data.draw_inertia_0(painter);
        data.draw_inertia_1(painter);
    }

    //affiche le centre de gravite
    pen.setWidth(6.0);
    pen.setColor(Qt::red);
    painter.setPen(pen);
    if(is_draw_inertia)
        data.draw_center(painter);

    //affiche la droite locale approximant
    pen.setWidth(3.0);
    pen.setColor(Qt:: magenta);
    painter.setPen(pen);
    if(is_draw_tangent)
        data.draw_projected_mls(painter);

}

//souris appuyee qui se deplace
void render_area::mouseMoveEvent(QMouseEvent *event)
{
    int x=event->x();
    int y=event->y();

    //mode ajout de sommet
    if(is_drawing_mode)
    {
        data.add_vertex(x,y);
        data.update_inertia();
    }
    //mode visualisation droite
    else if(is_click_mode)
        data.compute_line_approximation(x,y);

    repaint();
}

//simple click souris
void render_area::mousePressEvent(QMouseEvent *event)
{
    int x=event->x();
    int y=event->y();

    if(is_click_mode)
        data.compute_line_approximation(x,y);

    repaint();
}

//mise a jour des attributs d'affichage
void render_area::update_bool(bool is_drawing_mode_arg,
                              bool is_click_mode_arg,
                              bool is_draw_inertia_arg,
                              bool is_draw_tangent_arg)
{
    is_drawing_mode=is_drawing_mode_arg;
    is_click_mode=is_click_mode_arg;
    is_draw_inertia=is_draw_inertia_arg;
    is_draw_tangent=is_draw_tangent_arg;

    repaint();
}

